@@ -7,6 +7,7 @@ from tf_geometric.layers.conv.sgc import SGC
 from tf_geometric.layers.conv.tagcn import TAGCN
 from tf_geometric.layers.conv.chebynet import ChebyNet
 from tf_geometric.layers.conv.appnp import APPNP
+from tf_geometric.layers.conv.le_conv import LEConv
 
 from tf_geometric.layers.sampling.drop_edge import DropEdge
 
@@ -15,3 +16,6 @@ from tf_geometric.layers.kernel.map_reduce import MapReduceGNN
 from tf_geometric.layers.pool.common_pool import MeanPool, MinPool, MaxPool, SumPool
 from tf_geometric.layers.pool.diff_pool import DiffPool
 from tf_geometric.layers.pool.set2set import Set2Set
+from tf_geometric.layers.pool.sag_pool import SAGPool
+from tf_geometric.layers.pool.asap import ASAP
+from tf_geometric.layers.pool.sort_pool import SortPool
