@@ -10,6 +10,46 @@ from tf_geometric.utils.union_utils import convert_union_to_numpy
 from scipy.sparse.linalg import eigs, eigsh
 import scipy.sparse
 
+
+def convert_x_to_3d(x, source_index, K=None, pad=True):
+
+    source_index_perm = tf.argsort(source_index, stable=True)
+    sorted_source_index = tf.gather(source_index, source_index_perm)
+    permed_x = tf.gather(x, source_index_perm)
+
+    num_targets = tf.shape(sorted_source_index)[0]
+    target_ones = tf.ones([num_targets], dtype=tf.int32)
+    num_targets_for_sources = tf.math.segment_sum(target_ones, sorted_source_index)
+    max_num_targets_for_sources = tf.reduce_max(num_targets_for_sources)
+
+    # max index of source + 1
+    num_seen_sources = tf.shape(num_targets_for_sources)[0]
+
+    num_targets_before = tf.concat([
+        tf.zeros([1], dtype=tf.int32),
+        tf.math.cumsum(num_targets_for_sources)[:-1]
+    ], axis=0)
+
+    target_index_for_source = tf.range(0, num_targets) - tf.gather(num_targets_before, sorted_source_index)
+
+    if K is None:
+        K = max_num_targets_for_sources
+    elif K > max_num_targets_for_sources:
+        if not pad:
+            K = max_num_targets_for_sources
+    elif K < max_num_targets_for_sources:
+            mask = tf.less(target_index_for_source, K)
+            target_index_for_source = tf.boolean_mask(target_index_for_source, mask)
+            sorted_source_index = tf.boolean_mask(sorted_source_index, mask)
+            permed_x = tf.boolean_mask(permed_x, mask)
+
+
+    h = tf.zeros([num_seen_sources, K, tf.shape(x)[-1]], dtype=x.dtype)
+    index = tf.stack([sorted_source_index, target_index_for_source], axis=1)
+    h = tf.tensor_scatter_nd_update(h, index, permed_x)
+    return h
+
+
 def remove_self_loop_edge(edge_index, edge_weight=None):
     edge_index_is_tensor = tf.is_tensor(edge_index)
     edge_weight_is_tensor = tf.is_tensor(edge_weight)
@@ -37,6 +77,49 @@ def convert_dense_adj_to_edge(dense_adj):
     col = tf.reshape(col, [-1])
     edge_index = tf.stack([row, col], axis=0)
     edge_weight = tf.reshape(dense_adj, [-1])
+
+    edge_mask = tf.not_equal(edge_weight, 0.0)
+    edge_index = tf.boolean_mask(edge_index, edge_mask, axis=1)
+    edge_weight = tf.boolean_mask(edge_weight, edge_mask)
+
+    return edge_index, edge_weight
+
+
+def convert_dense_assign_to_edge(dense_assign, node_graph_index=None, num_nodes=None, num_clusters=None):
+    """
+    Convert a dense assignment matrix [num_nodes, num_clusters] of a Graph or BatchGraph to edge_index, edge_weight.
+    In the single-graph scenario (when node_graph_index is None), there are num_clusters clusters and 
+        the j_th column of each row correspond to the j_th clusters.
+    In the multi-graph scenario (where node_graph_index is not None), there are num_clusters * num_graphs clusters and
+        the j_th column of the i_th row correspond to the j_th cluster in the corresponding graph, and the cluster index
+        is num_clusters * node_graph_index[i] + j
+    
+    :param dense_assign: 
+    :param node_graph_index: 
+    :param num_nodes: 
+    :param num_clusters: 
+    :return: 
+    """
+    dense_shape = tf.shape(dense_assign)
+
+    # manually passing in num_row and num_col can boost the computation
+    if num_nodes is None:
+        num_nodes = dense_shape[0]
+    if num_clusters is None:
+        num_clusters = dense_shape[1]
+
+    row, col = tf.meshgrid(tf.range(num_nodes), tf.range(num_clusters), indexing="ij")
+
+    # multi graphs, the j_th clusters of the i_th graph is assigned a new cluster index: num_clusters * i + j
+    if node_graph_index is not None:
+        col += tf.expand_dims(node_graph_index, axis=-1) * num_clusters
+
+    row = tf.reshape(row, [-1])
+    col = tf.reshape(col, [-1])
+    edge_index = tf.stack([row, col], axis=0)
+
+    edge_weight = tf.reshape(dense_assign, [-1])
+
     return edge_index, edge_weight
 
 
@@ -89,7 +172,6 @@ def convert_edge_to_upper(edge_index, edge_properties=[]):
 
 # [[1,3,5], [2,1,4]] => [[1,3,5,2,1,4], [2,1,4,1,3,5]]
 def convert_edge_to_directed(edge_index, edge_properties=[]):
-
     edge_index_is_tensor = tf.is_tensor(edge_index)
     edge_properties_is_tensor = [tf.is_tensor(edge_property) for edge_property in edge_properties]
 
@@ -115,7 +197,6 @@ def convert_edge_to_directed(edge_index, edge_properties=[]):
 
 
 def add_self_loop_edge(edge_index, num_nodes, edge_weight=None, fill_weight=1.0):
-
     diagnal_edge_index = tf.stack([tf.range(num_nodes, dtype=tf.int32)] * 2, axis=0)
     updated_edge_index = tf.concat([edge_index, diagnal_edge_index], axis=1)
 
@@ -265,7 +346,8 @@ def edge_train_test_split(edge_index, test_size, edge_weight=None, mode="undirec
 
     # todo: warn user if they pass into "num_nodes", deprecated
     if "num_nodes" in kwargs:
-        warnings.warn("argument \"num_nodes\" is deprecated for the method \"edge_train_test_split\", you can remove it")
+        warnings.warn(
+            "argument \"num_nodes\" is deprecated for the method \"edge_train_test_split\", you can remove it")
 
     if mode == "undirected":
         is_edge_index_tensor = tf.is_tensor(edge_index)
@@ -303,30 +385,24 @@ def edge_train_test_split(edge_index, test_size, edge_weight=None, mode="undirec
 
 
 def compute_edge_mask_by_node_index(edge_index, node_index):
-
     edge_index_is_tensor = tf.is_tensor(edge_index)
 
-    node_index = convert_union_to_numpy(node_index)
-    edge_index = convert_union_to_numpy(edge_index)
-
-    max_node_index = np.maximum(np.max(edge_index), np.max(node_index))
-    node_mask = np.zeros([max_node_index + 1]).astype(np.bool)
-    node_mask[node_index] = True
-    row, col = edge_index
-    row_mask = node_mask[row]
-    col_mask = node_mask[col]
-    edge_mask = np.logical_and(row_mask, col_mask)
-
-    if edge_index_is_tensor:
-        edge_mask = tf.convert_to_tensor(edge_mask, dtype=tf.bool)
+    max_node_index = tf.maximum(tf.reduce_max(edge_index), tf.reduce_max(node_index))
+    node_mask = tf.scatter_nd(tf.expand_dims(node_index, axis=-1), tf.ones_like(node_index), [max_node_index + 1])
+    node_mask = tf.cast(node_mask, tf.bool)
+    row, col = edge_index[0], edge_index[1]
+    row_mask = tf.gather(node_mask, row)
+    col_mask = tf.gather(node_mask, col)
+    edge_mask = tf.logical_and(row_mask, col_mask)
 
+    if not edge_index_is_tensor:
+        edge_mask = edge_mask.numpy()
     return edge_mask
 
 
 def get_laplacian(edge_index, edge_weight, normalization_type, num_nodes, fill_weight=1.0):
-
     if normalization_type is not None:
-        assert normalization_type in [None,'sym', 'rw']
+        assert normalization_type in [None, 'sym', 'rw']
 
     row, col = edge_index[0], edge_index[1]
     deg = tf.math.unsorted_segment_sum(edge_weight, row, num_segments=num_nodes)
@@ -375,6 +451,7 @@ def get_laplacian(edge_index, edge_weight, normalization_type, num_nodes, fill_w
 
     return edge_index, edge_weight
 
+
 def to_scipy_sparse_matrix(edge_index, edge_weight=None, num_nodes=None):
     r"""Converts a graph given by edge indices and edge attributes to a scipy
     sparse matrix.
@@ -451,6 +528,7 @@ class RandomNeighborSampler(object):
         sampled_edge_weight = np.array(sampled_edge_weight)
         return sampled_edge_index, sampled_edge_weight
 
+
 class LaplacianMaxEigenvalue(object):
     def __init__(self, x, edge_index, edge_weight, is_undirected=True):
         self.num_nodes = x.shape[0]
@@ -461,7 +539,6 @@ class LaplacianMaxEigenvalue(object):
             self.edge_weight = np.ones([self.edge_index.shape[1]], dtype=np.float32)
         self.is_undirected = is_undirected
 
-
     def __call__(self, normalization_type='sym'):
         assert normalization_type in [None, 'sym', 'rw']
 
@@ -480,10 +557,3 @@ class LaplacianMaxEigenvalue(object):
         lambda_max = eig_fn(L, k=1, which='LM', return_eigenvectors=False)
 
         return float(lambda_max.real)
-
-
-
-
-
-
-
